@@ -20,7 +20,6 @@ import (
 	"context"
 	"fmt"
 	"io"
-	"os"
 	"strings"
 
 	"github.com/spf13/cobra"
@@ -31,6 +30,8 @@ import (
 	"k8s.io/kops/cmd/kops/util"
 	kopsapi "k8s.io/kops/pkg/apis/kops"
 	"k8s.io/kops/pkg/apis/kops/registry"
+	"k8s.io/kops/pkg/commands/commandutils"
+	"k8s.io/kops/pkg/kopscodecs"
 	"k8s.io/kops/util/pkg/tables"
 	"k8s.io/kubectl/pkg/util/i18n"
 	"k8s.io/kubectl/pkg/util/templates"
@@ -88,29 +89,26 @@ func NewCmdGetCluster(f *util.Factory, out io.Writer, getOptions *GetOptions) *c
 	}
 
 	cmd := &cobra.Command{
-		Use:     "clusters",
+		Use:     "clusters [CLUSTER]...",
 		Aliases: []string{"cluster"},
 		Short:   getClusterShort,
 		Long:    getClusterLong,
 		Example: getClusterExample,
-		Run: func(cmd *cobra.Command, args []string) {
-			ctx := context.TODO()
+		Args: func(cmd *cobra.Command, args []string) error {
 			if len(args) != 0 {
-				options.ClusterNames = append(options.ClusterNames, args...)
-			}
-
-			if rootCommand.clusterName != "" {
-				if len(args) != 0 {
-					exitWithError(fmt.Errorf("cannot mix --name for cluster with positional arguments"))
+				if rootCommand.clusterName != "" {
+					return fmt.Errorf("cannot mix --name for cluster with positional arguments")
 				}
-
+				options.ClusterNames = append(options.ClusterNames, args...)
+			} else if rootCommand.clusterName != "" {
 				options.ClusterNames = append(options.ClusterNames, rootCommand.clusterName)
 			}
 
-			err := RunGetClusters(ctx, &rootCommand, os.Stdout, &options)
-			if err != nil {
-				exitWithError(err)
-			}
+			return nil
+		},
+		ValidArgsFunction: commandutils.CompleteClusterName(f, false, true),
+		RunE: func(cmd *cobra.Command, args []string) error {
+			return RunGetClusters(context.TODO(), f, out, &options)
 		},
 	}
 
@@ -119,7 +117,7 @@ func NewCmdGetCluster(f *util.Factory, out io.Writer, getOptions *GetOptions) *c
 	return cmd
 }
 
-func RunGetClusters(ctx context.Context, f Factory, out io.Writer, options *GetClusterOptions) error {
+func RunGetClusters(ctx context.Context, f commandutils.Factory, out io.Writer, options *GetClusterOptions) error {
 	client, err := f.Clientset()
 	if err != nil {
 		return err
@@ -166,13 +164,13 @@ func RunGetClusters(ctx context.Context, f Factory, out io.Writer, options *GetC
 	}
 
 	var obj []runtime.Object
-	if options.output != OutputTable {
+	if options.Output != OutputTable {
 		for _, c := range clusters {
 			obj = append(obj, c)
 		}
 	}
 
-	switch options.output {
+	switch options.Output {
 	case OutputTable:
 		return clusterOutputTable(clusters, out)
 	case OutputYaml:
@@ -180,7 +178,7 @@ func RunGetClusters(ctx context.Context, f Factory, out io.Writer, options *GetC
 	case OutputJSON:
 		return fullOutputJSON(out, obj...)
 	default:
-		return fmt.Errorf("Unknown output format: %q", options.output)
+		return fmt.Errorf("Unknown output format: %q", options.Output)
 	}
 }
 
@@ -283,12 +281,21 @@ func fullClusterSpecs(clusters []*kopsapi.Cluster) ([]*kopsapi.Cluster, error) {
 		if err != nil {
 			return nil, fmt.Errorf("error reading full cluster spec for %q: %v", cluster.ObjectMeta.Name, err)
 		}
-		fullSpec := &kopsapi.Cluster{}
-		err = registry.ReadConfigDeprecated(configBase.Join(registry.PathClusterCompleted), fullSpec)
+		configPath := configBase.Join(registry.PathClusterCompleted)
+		b, err := configPath.ReadFile()
 		if err != nil {
-			return nil, fmt.Errorf("error reading full cluster spec for %q: %v", cluster.ObjectMeta.Name, err)
+			return nil, fmt.Errorf("error loading Cluster %q: %v", configPath, err)
+		}
+
+		o, _, err := kopscodecs.Decode(b, nil)
+		if err != nil {
+			return nil, fmt.Errorf("error parsing Cluster %q: %v", configPath, err)
+		}
+		if fullSpec, ok := o.(*kopsapi.Cluster); ok {
+			fullSpecs = append(fullSpecs, fullSpec)
+		} else {
+			return nil, fmt.Errorf("unexpected object type for Cluster %q: %T", configPath, o)
 		}
-		fullSpecs = append(fullSpecs, fullSpec)
 	}
 	return fullSpecs, nil
 }
