@@ -23,6 +23,7 @@ import (
 	"strings"
 
 	"github.com/blang/semver/v4"
+	"github.com/pelletier/go-toml"
 	"k8s.io/klog/v2"
 	"k8s.io/kops/nodeup/pkg/model/resources"
 	"k8s.io/kops/pkg/apis/kops"
@@ -77,7 +78,9 @@ func (b *ContainerdBuilder) Build(c *fi.ModelBuilderContext) error {
 	}
 
 	// If there are containerd configuration overrides, apply them
-	b.buildOverrideConfigFile(c)
+	if err := b.buildConfigFile(c); err != nil {
+		return err
+	}
 
 	if installContainerd {
 		if err := b.installContainerd(c); err != nil {
@@ -127,7 +130,7 @@ func (b *ContainerdBuilder) installContainerd(c *fi.ModelBuilderContext) error {
 	var containerRuntimeVersion string
 	if b.Cluster.Spec.ContainerRuntime == "containerd" {
 		if b.Cluster.Spec.Containerd != nil {
-			containerRuntimeVersion = fi.StringValue(b.Cluster.Spec.Containerd.Version)
+			containerRuntimeVersion = fi.StringValue(b.NodeupConfig.ContainerdConfig.Version)
 		} else {
 			return fmt.Errorf("error finding contained version")
 		}
@@ -275,8 +278,8 @@ func (b *ContainerdBuilder) buildSystemdServiceOverrideFlatcar(c *fi.ModelBuilde
 // buildSysconfigFile is responsible for creating the containerd sysconfig file
 func (b *ContainerdBuilder) buildSysconfigFile(c *fi.ModelBuilderContext) error {
 	var containerd kops.ContainerdConfig
-	if b.Cluster.Spec.Containerd != nil {
-		containerd = *b.Cluster.Spec.Containerd
+	if b.NodeupConfig.ContainerdConfig != nil {
+		containerd = *b.NodeupConfig.ContainerdConfig
 	}
 
 	flagsString, err := flagbuilder.BuildFlags(&containerd)
@@ -298,23 +301,30 @@ func (b *ContainerdBuilder) buildSysconfigFile(c *fi.ModelBuilderContext) error
 	return nil
 }
 
-// buildOverrideConfigFile is responsible for creating the containerd configuration file
-func (b *ContainerdBuilder) buildOverrideConfigFile(c *fi.ModelBuilderContext) {
-	containerdConfigOverride := ""
-	if b.Cluster.Spec.Containerd != nil {
-		containerdConfigOverride = fi.StringValue(b.Cluster.Spec.Containerd.ConfigOverride)
-	}
+// buildConfigFile is responsible for creating the containerd configuration file
+func (b *ContainerdBuilder) buildConfigFile(c *fi.ModelBuilderContext) error {
+	var config string
 
+	if b.NodeupConfig.ContainerdConfig != nil && b.NodeupConfig.ContainerdConfig.ConfigOverride != nil {
+		config = fi.StringValue(b.NodeupConfig.ContainerdConfig.ConfigOverride)
+	} else {
+		if cc, err := b.buildContainerdConfig(); err != nil {
+			return err
+		} else {
+			config = cc
+		}
+	}
 	c.AddTask(&nodetasks.File{
 		Path:     b.containerdConfigFilePath(),
-		Contents: fi.NewStringResource(containerdConfigOverride),
+		Contents: fi.NewStringResource(config),
 		Type:     nodetasks.FileType_File,
 	})
+	return nil
 }
 
 // skipInstall determines if kops should skip the installation and configuration of containerd
 func (b *ContainerdBuilder) skipInstall() bool {
-	d := b.Cluster.Spec.Containerd
+	d := b.NodeupConfig.ContainerdConfig
 
 	// don't skip install if the user hasn't specified anything
 	if d == nil {
@@ -426,3 +436,64 @@ func (b *ContainerdBuilder) buildCNIConfigTemplateFile(c *fi.ModelBuilderContext
 		Type:     nodetasks.FileType_File,
 	})
 }
+
+func (b *ContainerdBuilder) buildContainerdConfig() (string, error) {
+	cluster := b.Cluster
+
+	if cluster.Spec.ContainerRuntime != "containerd" {
+		return "", nil
+	}
+
+	containerd := b.NodeupConfig.ContainerdConfig
+	if fi.StringValue(containerd.ConfigOverride) != "" {
+		return *containerd.ConfigOverride, nil
+	}
+
+	// Build config file for containerd running in CRI mode
+
+	config, _ := toml.Load("")
+	config.SetPath([]string{"version"}, int64(2))
+	for name, endpoints := range containerd.RegistryMirrors {
+		config.SetPath([]string{"plugins", "io.containerd.grpc.v1.cri", "registry", "mirrors", name, "endpoint"}, endpoints)
+	}
+	config.SetPath([]string{"plugins", "io.containerd.grpc.v1.cri", "containerd", "runtimes", "runc", "runtime_type"}, "io.containerd.runc.v2")
+	// only enable systemd cgroups for kubernetes >= 1.20
+	config.SetPath([]string{"plugins", "io.containerd.grpc.v1.cri", "containerd", "runtimes", "runc", "options", "SystemdCgroup"}, cluster.IsKubernetesGTE("1.20"))
+	if components.UsesKubenet(cluster.Spec.Networking) {
+		// Using containerd with Kubenet requires special configuration.
+		// This is a temporary backwards-compatible solution for kubenet users and will be deprecated when Kubenet is deprecated:
+		// https://github.com/containerd/containerd/blob/master/docs/cri/config.md#cni-config-template
+		config.SetPath([]string{"plugins", "io.containerd.grpc.v1.cri", "cni", "conf_template"}, "/etc/containerd/config-cni.template")
+	}
+
+	if b.InstallNvidiaRuntime() {
+		if err := appendNvidiaGPURuntimeConfig(config); err != nil {
+			return "", err
+		}
+	}
+	return config.String(), nil
+}
+
+func appendNvidiaGPURuntimeConfig(config *toml.Tree) error {
+
+	gpuConfig, err := toml.TreeFromMap(
+		map[string]interface{}{
+			"privileged_without_host_devices": false,
+			"runtime_engine":                  "",
+			"runtime_root":                    "",
+			"runtime_type":                    "io.containerd.runc.v1",
+			"options": map[string]interface{}{
+				"SystemdCgroup": true,
+				"BinaryName":    "/usr/bin/nvidia-container-runtime",
+			},
+		},
+	)
+	if err != nil {
+		return err
+	}
+
+	config.SetPath([]string{"plugins", "io.containerd.grpc.v1.cri", "containerd", "runtimes", "nvidia"}, gpuConfig)
+	config.SetPath([]string{"plugins", "io.containerd.grpc.v1.cri", "containerd", "default_runtime_name"}, "runc")
+
+	return nil
+}
