@@ -33,26 +33,26 @@ type Config struct {
 	Assets map[architectures.Architecture][]string `json:",omitempty"`
 	// Images are a list of images we should preload
 	Images map[architectures.Architecture][]*Image `json:"images,omitempty"`
-	// ConfigBase is the base VFS path for config objects
-	ConfigBase *string `json:",omitempty"`
-	// ClusterLocation is the VFS path to the cluster spec (deprecated: prefer ConfigBase)
-	ClusterLocation *string `json:",omitempty"`
-	// InstanceGroupName is the name of the instance group
-	InstanceGroupName string `json:",omitempty"`
-	// InstanceGroupRole is the instance group role.
-	InstanceGroupRole kops.InstanceGroupRole
 	// ClusterName is the name of the cluster
 	ClusterName string `json:",omitempty"`
 	// Channels is a list of channels that we should apply
 	Channels []string `json:"channels,omitempty"`
 	// ApiserverAdditionalIPs are additional IP address to put in the apiserver server cert.
 	ApiserverAdditionalIPs []string `json:",omitempty"`
+	// WarmPoolImages are the container images to pre-pull during instance pre-initialization
+	WarmPoolImages []string `json:"warmPoolImages,omitempty"`
 
 	// Manifests for running etcd
 	EtcdManifests []string `json:"etcdManifests,omitempty"`
 
+	// CAs are the CA certificates to trust.
+	CAs map[string]string
+	// KeypairIDs are the IDs of keysets used to sign things.
+	KeypairIDs map[string]string
 	// DefaultMachineType is the first-listed instance machine type, used if querying instance metadata fails.
 	DefaultMachineType *string `json:",omitempty"`
+	// EnableLifecycleHook defines whether we need to complete a lifecycle hook.
+	EnableLifecycleHook bool `json:",omitempty"`
 	// StaticManifests describes generic static manifests
 	// Using this allows us to keep complex logic out of nodeup
 	StaticManifests []*StaticManifest `json:"staticManifests,omitempty"`
@@ -67,18 +67,40 @@ type Config struct {
 	// VolumeMounts are a collection of volume mounts.
 	VolumeMounts []kops.VolumeMountSpec `json:",omitempty"`
 
-	// ConfigServer holds the configuration for the configuration server
-	ConfigServer *ConfigServerOptions `json:"configServer,omitempty"`
+	// FileAssets are a collection of file assets for this instance group.
+	FileAssets []kops.FileAssetSpec `json:",omitempty"`
+	// Hooks are for custom actions, for example on first installation.
+	Hooks [][]kops.HookSpec
+	// ContainerdConfig config holds the configuration for containerd
+	ContainerdConfig *kops.ContainerdConfig `json:"containerdConfig,omitempty"`
+
+	// APIServerConfig is additional configuration for nodes running an APIServer.
+	APIServerConfig *APIServerConfig `json:",omitempty"`
+	// NvidiaGPU contains the configuration for nvidia
+	NvidiaGPU *kops.NvidiaGPUConfig `json:",omitempty"`
+}
+
+// BootConfig is the configuration for the nodeup binary that might be too big to fit in userdata.
+type BootConfig struct {
+	// CloudProvider is the cloud provider in use.
+	CloudProvider string
+	// ConfigBase is the base VFS path for config objects.
+	ConfigBase *string `json:",omitempty"`
+	// ConfigServer holds the configuration for the configuration server.
+	ConfigServer *ConfigServerOptions `json:",omitempty"`
+	// InstanceGroupName is the name of the instance group.
+	InstanceGroupName string `json:",omitempty"`
+	// InstanceGroupRole is the instance group role.
+	InstanceGroupRole kops.InstanceGroupRole
+	// NodeupConfigHash holds a secure hash of the nodeup.Config.
+	NodeupConfigHash string
 }
 
 type ConfigServerOptions struct {
 	// Server is the address of the configuration server to use (kops-controller)
 	Server string `json:"server,omitempty"`
-	// CA is the ca-certificate to require for the configuration server
-	CA string `json:"ca,omitempty"`
-
-	// CloudProvider is the cloud provider in use (needed for authentication)
-	CloudProvider string `json:"cloudProvider,omitempty"`
+	// CACertificates are the certificates to trust for fi.CertificateIDCA.
+	CACertificates string
 }
 
 // Image is a docker image we should pre-load
@@ -99,14 +121,44 @@ type StaticManifest struct {
 	Path string `json:"path,omitempty"`
 }
 
-func NewConfig(cluster *kops.Cluster, instanceGroup *kops.InstanceGroup) *Config {
+// APIServerConfig is additional configuration for nodes running an APIServer.
+type APIServerConfig struct {
+	// KubeAPIServer is a copy of the KubeAPIServerConfig from the cluster spec.
+	KubeAPIServer *kops.KubeAPIServerConfig
+	// EncryptionConfigSecretHash is a hash of the encryptionconfig secret.
+	// It is empty if EncryptionConfig is not enabled.
+	// TODO: give secrets IDs and look them up like we do keypairs.
+	EncryptionConfigSecretHash string `json:",omitempty"`
+	// ServiceAccountPublicKeys are the service-account public keys to trust.
+	ServiceAccountPublicKeys string
+}
+
+func NewConfig(cluster *kops.Cluster, instanceGroup *kops.InstanceGroup) (*Config, *BootConfig) {
 	role := instanceGroup.Spec.Role
 	isMaster := role == kops.InstanceGroupRoleMaster
 
+	clusterHooks := filterHooks(cluster.Spec.Hooks, instanceGroup.Spec.Role)
+	igHooks := filterHooks(instanceGroup.Spec.Hooks, instanceGroup.Spec.Role)
+
 	config := Config{
+		ClusterName:      cluster.ObjectMeta.Name,
+		CAs:              map[string]string{},
+		KeypairIDs:       map[string]string{},
+		SysctlParameters: instanceGroup.Spec.SysctlParameters,
+		VolumeMounts:     instanceGroup.Spec.VolumeMounts,
+		FileAssets:       append(filterFileAssets(instanceGroup.Spec.FileAssets, role), filterFileAssets(cluster.Spec.FileAssets, role)...),
+		Hooks:            [][]kops.HookSpec{igHooks, clusterHooks},
+	}
+
+	bootConfig := BootConfig{
+		CloudProvider:     cluster.Spec.CloudProvider,
+		InstanceGroupName: instanceGroup.ObjectMeta.Name,
 		InstanceGroupRole: role,
-		SysctlParameters:  instanceGroup.Spec.SysctlParameters,
-		VolumeMounts:      instanceGroup.Spec.VolumeMounts,
+	}
+
+	warmPool := cluster.Spec.WarmPool.ResolveDefaults(instanceGroup)
+	if warmPool.IsEnabled() && warmPool.EnableLifecycleHook {
+		config.EnableLifecycleHook = true
 	}
 
 	if isMaster {
@@ -120,6 +172,12 @@ func NewConfig(cluster *kops.Cluster, instanceGroup *kops.InstanceGroup) *Config
 		reflectutils.JSONMergeStruct(&config.KubeletConfig, cluster.Spec.Kubelet)
 	}
 
+	if isMaster || role == kops.InstanceGroupRoleAPIServer {
+		config.APIServerConfig = &APIServerConfig{
+			KubeAPIServer: cluster.Spec.KubeAPIServer,
+		}
+	}
+
 	if instanceGroup.Spec.Kubelet != nil {
 		useSecureKubelet := config.KubeletConfig.AnonymousAuth != nil && !*config.KubeletConfig.AnonymousAuth
 
@@ -148,5 +206,39 @@ func NewConfig(cluster *kops.Cluster, instanceGroup *kops.InstanceGroup) *Config
 		config.DefaultMachineType = fi.String(strings.Split(instanceGroup.Spec.MachineType, ",")[0])
 	}
 
-	return &config
+	return &config, &bootConfig
+}
+
+func filterFileAssets(f []kops.FileAssetSpec, role kops.InstanceGroupRole) []kops.FileAssetSpec {
+	var fileAssets []kops.FileAssetSpec
+	for _, fileAsset := range f {
+		if len(fileAsset.Roles) > 0 && !containsRole(role, fileAsset.Roles) {
+			continue
+		}
+		fileAsset.Roles = nil
+		fileAssets = append(fileAssets, fileAsset)
+	}
+	return fileAssets
+}
+
+func filterHooks(h []kops.HookSpec, role kops.InstanceGroupRole) []kops.HookSpec {
+	var hooks []kops.HookSpec
+	for _, hook := range h {
+		if len(hook.Roles) > 0 && !containsRole(role, hook.Roles) {
+			continue
+		}
+		hook.Roles = nil
+		hooks = append(hooks, hook)
+	}
+	return hooks
+}
+
+func containsRole(v kops.InstanceGroupRole, list []kops.InstanceGroupRole) bool {
+	for _, x := range list {
+		if v == x {
+			return true
+		}
+	}
+
+	return false
 }
