@@ -33,7 +33,6 @@ import (
 	"sort"
 	"strings"
 
-	"github.com/aws/aws-sdk-go/aws/endpoints"
 	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/klog/v2"
 	"k8s.io/kops/pkg/apis/kops"
@@ -247,6 +246,7 @@ type PolicyBuilder struct {
 	HostedZoneID                          string
 	KMSKeys                               []string
 	Region                                string
+	Partition                             string
 	ResourceARN                           *string
 	Role                                  Subject
 	UseServiceAccountExternalPermisssions bool
@@ -285,7 +285,7 @@ func NewPolicy(clusterName string) *Policy {
 
 // BuildAWSPolicy generates a custom policy for a Kubernetes master.
 func (r *NodeRoleAPIServer) BuildAWSPolicy(b *PolicyBuilder) (*Policy, error) {
-	p := NewPolicy(b.Cluster.GetClusterName())
+	p := NewPolicy(b.Cluster.GetName())
 
 	addNodeupPermissions(p, r.warmPool)
 
@@ -303,7 +303,7 @@ func (r *NodeRoleAPIServer) BuildAWSPolicy(b *PolicyBuilder) (*Policy, error) {
 	}
 
 	if b.Cluster.Spec.Networking != nil && b.Cluster.Spec.Networking.AmazonVPC != nil {
-		addAmazonVPCCNIPermissions(p, b.IAMPrefix())
+		addAmazonVPCCNIPermissions(p, b.Partition)
 	}
 
 	if b.Cluster.Spec.Networking != nil && b.Cluster.Spec.Networking.LyftVPC != nil {
@@ -344,17 +344,17 @@ func (r *NodeRoleMaster) BuildAWSPolicy(b *PolicyBuilder) (*Policy, error) {
 
 	// If cluster does not use external CCM, the master IAM Role needs CCM permissions
 	if b.Cluster.Spec.ExternalCloudControllerManager == nil {
-		AddCCMPermissions(p, b.Cluster.Spec.Networking.Kubenet != nil)
+		AddCCMPermissions(p, b.Partition, b.Cluster.Spec.Networking.Kubenet != nil)
 		AddLegacyCCMPermissions(p)
 	}
 
 	if !b.UseServiceAccountExternalPermisssions {
 		esc := b.Cluster.Spec.SnapshotController != nil &&
 			fi.BoolValue(b.Cluster.Spec.SnapshotController.Enabled)
-		AddAWSEBSCSIDriverPermissions(p, esc)
+		AddAWSEBSCSIDriverPermissions(p, b.Partition, esc)
 
 		if b.Cluster.Spec.ExternalCloudControllerManager != nil {
-			AddCCMPermissions(p, b.Cluster.Spec.Networking.Kubenet != nil)
+			AddCCMPermissions(p, b.Partition, b.Cluster.Spec.Networking.Kubenet != nil)
 			AddLegacyCCMPermissions(p)
 		}
 
@@ -374,7 +374,7 @@ func (r *NodeRoleMaster) BuildAWSPolicy(b *PolicyBuilder) (*Policy, error) {
 	}
 
 	if b.Cluster.Spec.Networking != nil && b.Cluster.Spec.Networking.AmazonVPC != nil {
-		addAmazonVPCCNIPermissions(p, b.IAMPrefix())
+		addAmazonVPCCNIPermissions(p, b.Partition)
 	}
 
 	if b.Cluster.Spec.Networking != nil && b.Cluster.Spec.Networking.LyftVPC != nil {
@@ -394,7 +394,7 @@ func (r *NodeRoleMaster) BuildAWSPolicy(b *PolicyBuilder) (*Policy, error) {
 
 // BuildAWSPolicy generates a custom policy for a Kubernetes node.
 func (r *NodeRoleNode) BuildAWSPolicy(b *PolicyBuilder) (*Policy, error) {
-	p := NewPolicy(b.Cluster.GetClusterName())
+	p := NewPolicy(b.Cluster.GetName())
 
 	addNodeupPermissions(p, r.enableLifecycleHookPermissions)
 
@@ -408,7 +408,7 @@ func (r *NodeRoleNode) BuildAWSPolicy(b *PolicyBuilder) (*Policy, error) {
 	}
 
 	if b.Cluster.Spec.Networking != nil && b.Cluster.Spec.Networking.AmazonVPC != nil {
-		addAmazonVPCCNIPermissions(p, b.IAMPrefix())
+		addAmazonVPCCNIPermissions(p, b.Partition)
 	}
 
 	if b.Cluster.Spec.Networking != nil && b.Cluster.Spec.Networking.LyftVPC != nil {
@@ -424,7 +424,7 @@ func (r *NodeRoleNode) BuildAWSPolicy(b *PolicyBuilder) (*Policy, error) {
 
 // BuildAWSPolicy generates a custom policy for a bastion host.
 func (r *NodeRoleBastion) BuildAWSPolicy(b *PolicyBuilder) (*Policy, error) {
-	p := NewPolicy(b.Cluster.GetClusterName())
+	p := NewPolicy(b.Cluster.GetName())
 
 	// Bastion hosts currently don't require any specific permissions.
 	// A trivial permission is granted, because empty policies are not allowed.
@@ -433,19 +433,6 @@ func (r *NodeRoleBastion) BuildAWSPolicy(b *PolicyBuilder) (*Policy, error) {
 	return p, nil
 }
 
-// IAMPrefix returns the prefix for AWS ARNs in the current region, for use with IAM
-// it is arn:aws in the default aws partition but different in other isolated or non-standard partitions
-func (b *PolicyBuilder) IAMPrefix() string {
-	partitions := endpoints.DefaultPartitions()
-	for _, p := range partitions {
-		if _, ok := p.Regions()[b.Region]; ok {
-			arn := "arn:" + p.ID()
-			return arn
-		}
-	}
-	return "arn:aws"
-}
-
 // AddS3Permissions builds an IAM Policy, with statements granting tailored
 // access to S3 assets, depending on the instance group or service-account role
 func (b *PolicyBuilder) AddS3Permissions(p *Policy) (*Policy, error) {
@@ -568,7 +555,7 @@ func (b *PolicyBuilder) AddS3Permissions(p *Policy) (*Policy, error) {
 				"s3:ListBucketVersions",
 			),
 			Resource: stringorslice.Slice([]string{
-				strings.Join([]string{b.IAMPrefix(), ":s3:::", s3Bucket}, ""),
+				fmt.Sprintf("arn:%v:s3:::%v", b.Partition, s3Bucket),
 			}),
 		})
 	}
@@ -586,7 +573,7 @@ func (b *PolicyBuilder) buildS3WriteStatements(p *Policy, iamS3Path string) {
 			"s3:PutObject",
 		}),
 		Resource: stringorslice.Of(
-			strings.Join([]string{b.IAMPrefix(), ":s3:::", iamS3Path, "/*"}, ""),
+			fmt.Sprintf("arn:%v:s3:::%v/*", b.Partition, iamS3Path),
 		),
 	})
 
@@ -604,7 +591,7 @@ func (b *PolicyBuilder) buildS3GetStatements(p *Policy, iamS3Path string) error
 
 		// Add the prefix for IAM
 		for i, r := range resources {
-			resources[i] = b.IAMPrefix() + ":s3:::" + iamS3Path + r
+			resources[i] = fmt.Sprintf("arn:%v:s3:::%v%v", b.Partition, iamS3Path, r)
 		}
 
 		p.Statement = append(p.Statement, &Statement{
@@ -808,10 +795,14 @@ func AddLegacyCCMPermissions(p *Policy) {
 	p.unconditionalAction.Insert(
 		"ec2:CreateSecurityGroup",
 		"ec2:CreateTags",
+		"elasticloadbalancing:CreateTargetGroup",
+		"elasticloadbalancing:AddTags",
+		"elasticloadbalancing:RegisterTargets",
+		"elasticloadbalancing:CreateListener",
 	)
 }
 
-func AddCCMPermissions(p *Policy, cloudRoutes bool) {
+func AddCCMPermissions(p *Policy, partition string, cloudRoutes bool) {
 	p.unconditionalAction.Insert(
 		"autoscaling:DescribeAutoScalingGroups",
 		"autoscaling:DescribeTags",
@@ -844,6 +835,8 @@ func AddCCMPermissions(p *Policy, cloudRoutes bool) {
 		"elasticloadbalancing:AddTags",
 		"elasticloadbalancing:AttachLoadBalancerToSubnets",
 		"elasticloadbalancing:ApplySecurityGroupsToLoadBalancer",
+		"elasticloadbalancing:CreateLoadBalancerListeners",
+		"elasticloadbalancing:CreateLoadBalancerPolicy",
 		"elasticloadbalancing:ConfigureHealthCheck",
 		"elasticloadbalancing:DeleteLoadBalancer",
 		"elasticloadbalancing:DeleteLoadBalancerListeners",
@@ -864,8 +857,6 @@ func AddCCMPermissions(p *Policy, cloudRoutes bool) {
 
 	p.clusterTaggedCreateAction.Insert(
 		"elasticloadbalancing:CreateLoadBalancer",
-		"elasticloadbalancing:CreateLoadBalancerPolicy",
-		"elasticloadbalancing:CreateLoadBalancerListeners",
 		"ec2:CreateSecurityGroup",
 		"ec2:CreateVolume",
 		"elasticloadbalancing:CreateListener",
@@ -880,8 +871,8 @@ func AddCCMPermissions(p *Policy, cloudRoutes bool) {
 			),
 			Resource: stringorslice.Slice(
 				[]string{
-					"arn:aws:ec2:*:*:volume/*",
-					"arn:aws:ec2:*:*:snapshot/*",
+					fmt.Sprintf("arn:%v:ec2:*:*:volume/*", partition),
+					fmt.Sprintf("arn:%v:ec2:*:*:snapshot/*", partition),
 				},
 			),
 			Condition: Condition{
@@ -913,30 +904,22 @@ func AddAWSLoadbalancerControllerPermissions(p *Policy) {
 		"elasticloadbalancing:DescribeTargetHealth",
 		"elasticloadbalancing:DescribeListenerCertificates",
 		"elasticloadbalancing:CreateRule",
+		"acm:ListCertificates",
+		"acm:DescribeCertificate",
 	)
-	p.Statement = append(p.Statement,
-		&Statement{
-			Effect: StatementEffectAllow,
-			Action: stringorslice.Of(
-				"ec2:AuthorizeSecurityGroupIngress", // aws.go
-				"ec2:DeleteSecurityGroup",           // aws.go
-				"ec2:RevokeSecurityGroupIngress",    // aws.go
+	p.clusterTaggedAction.Insert(
+		"ec2:AuthorizeSecurityGroupIngress", // aws.go
+		"ec2:DeleteSecurityGroup",           // aws.go
+		"ec2:RevokeSecurityGroupIngress",    // aws.go
 
-				"elasticloadbalancing:ModifyTargetGroupAttributes",
-				"elasticloadbalancing:ModifyRule",
-				"elasticloadbalancing:DeleteRule",
+		"elasticloadbalancing:ModifyTargetGroupAttributes",
+		"elasticloadbalancing:ModifyRule",
+		"elasticloadbalancing:DeleteRule",
 
-				"elasticloadbalancing:AddTags",
-				"elasticloadbalancing:RemoveTags",
-			),
-			Resource: stringorslice.String("*"),
-			Condition: Condition{
-				"StringEquals": map[string]string{
-					"aws:ResourceTag/elbv2.k8s.aws/cluster": p.clusterName,
-				},
-			},
-		},
+		"elasticloadbalancing:AddTags",
+		"elasticloadbalancing:RemoveTags",
 	)
+
 }
 
 func AddClusterAutoscalerPermissions(p *Policy) {
@@ -953,7 +936,7 @@ func AddClusterAutoscalerPermissions(p *Policy) {
 }
 
 // AddAWSEBSCSIDriverPermissions appens policy statements that the AWS EBS CSI Driver needs to operate.
-func AddAWSEBSCSIDriverPermissions(p *Policy, appendSnapshotPermissions bool) {
+func AddAWSEBSCSIDriverPermissions(p *Policy, partition string, appendSnapshotPermissions bool) {
 
 	if appendSnapshotPermissions {
 		addSnapshotPersmissions(p)
@@ -985,8 +968,8 @@ func AddAWSEBSCSIDriverPermissions(p *Policy, appendSnapshotPermissions bool) {
 			),
 			Resource: stringorslice.Slice(
 				[]string{
-					"arn:aws:ec2:*:*:volume/*",
-					"arn:aws:ec2:*:*:snapshot/*",
+					fmt.Sprintf("arn:%v:ec2:*:*:volume/*", partition),
+					fmt.Sprintf("arn:%v:ec2:*:*:snapshot/*", partition),
 				},
 			),
 			Condition: Condition{
@@ -1006,8 +989,8 @@ func AddAWSEBSCSIDriverPermissions(p *Policy, appendSnapshotPermissions bool) {
 			),
 			Resource: stringorslice.Slice(
 				[]string{
-					"arn:aws:ec2:*:*:volume/*",
-					"arn:aws:ec2:*:*:snapshot/*",
+					fmt.Sprintf("arn:%v:ec2:*:*:volume/*", partition),
+					fmt.Sprintf("arn:%v:ec2:*:*:snapshot/*", partition),
 				},
 			),
 			Condition: Condition{
@@ -1048,13 +1031,13 @@ func AddDNSControllerPermissions(b *PolicyBuilder, p *Policy) {
 		Action: stringorslice.Of("route53:ChangeResourceRecordSets",
 			"route53:ListResourceRecordSets",
 			"route53:GetHostedZone"),
-		Resource: stringorslice.Slice([]string{b.IAMPrefix() + ":route53:::hostedzone/" + hostedZoneID}),
+		Resource: stringorslice.Slice([]string{fmt.Sprintf("arn:%v:route53:::hostedzone/%v", b.Partition, hostedZoneID)}),
 	})
 
 	p.Statement = append(p.Statement, &Statement{
 		Effect:   StatementEffectAllow,
 		Action:   stringorslice.Slice([]string{"route53:GetChange"}),
-		Resource: stringorslice.Slice([]string{b.IAMPrefix() + ":route53:::change/*"}),
+		Resource: stringorslice.Slice([]string{fmt.Sprintf("arn:%v:route53:::change/*", b.Partition)}),
 	})
 
 	wildcard := stringorslice.Slice([]string{"*"})
@@ -1141,7 +1124,7 @@ func addCiliumEniPermissions(p *Policy) {
 	)
 }
 
-func addAmazonVPCCNIPermissions(p *Policy, iamPrefix string) {
+func addAmazonVPCCNIPermissions(p *Policy, partition string) {
 	p.unconditionalAction.Insert(
 		"ec2:AssignPrivateIpAddresses",
 		"ec2:AttachNetworkInterface",
@@ -1162,7 +1145,7 @@ func addAmazonVPCCNIPermissions(p *Policy, iamPrefix string) {
 				"ec2:CreateTags",
 			}),
 			Resource: stringorslice.Slice([]string{
-				strings.Join([]string{iamPrefix, ":ec2:*:*:network-interface/*"}, ""),
+				strings.Join([]string{"arn:", partition, ":ec2:*:*:network-interface/*"}, ""),
 			})},
 	)
 }
