@@ -17,7 +17,6 @@ limitations under the License.
 package cloudup
 
 import (
-	"encoding/binary"
 	"fmt"
 	"net"
 	"strings"
@@ -280,7 +279,9 @@ func (c *populateClusterSpec) run(clientset simple.Clientset) error {
 			codeModels = append(codeModels, &components.DiscoveryOptionsBuilder{OptionsContext: optionsContext})
 			codeModels = append(codeModels, &components.ClusterAutoscalerOptionsBuilder{OptionsContext: optionsContext})
 			codeModels = append(codeModels, &components.NodeTerminationHandlerOptionsBuilder{OptionsContext: optionsContext})
+			codeModels = append(codeModels, &components.NodeProblemDetectorOptionsBuilder{OptionsContext: optionsContext})
 			codeModels = append(codeModels, &components.AWSEBSCSIDriverOptionsBuilder{OptionsContext: optionsContext})
+			codeModels = append(codeModels, &components.AWSCloudControllerManagerOptionsBuilder{OptionsContext: optionsContext})
 		}
 	}
 
@@ -325,28 +326,29 @@ func (c *populateClusterSpec) assignSubnets(cluster *kopsapi.Cluster) error {
 		cluster.Spec.KubeControllerManager = &kopsapi.KubeControllerManagerConfig{}
 	}
 
-	if cluster.Spec.KubeControllerManager.ClusterCIDR == "" {
+	if cluster.Spec.PodCIDR == "" {
 		// Allocate as big a range as possible: the NonMasqueradeCIDR mask + 1, with a '1' in the extra bit
 		ip := nonMasqueradeCIDR.IP.Mask(nonMasqueradeCIDR.Mask)
-
-		ip4 := ip.To4()
-		if ip4 != nil {
-			n := binary.BigEndian.Uint32(ip4)
-			n += uint32(1 << uint(nmBits-nmOnes-1))
-			ip = make(net.IP, len(ip4))
-			binary.BigEndian.PutUint32(ip, n)
-		} else {
-			return fmt.Errorf("IPV6 subnet computations not yet implements")
+		if nmBits > 32 && nmOnes < 95 {
+			// The maximum size of an IPv6 ClusterCIDR is /64, but a /112 node CIDR gives far more addresses
+			// than Kubernetes can handle on a node and is more visually pleasing.
+			// Technically, the maximum size of an IPv4 ClusterCIDR is /8, but nobody has a /7 to allocate.
+			nmOnes = 95
 		}
-
+		ip[nmOnes/8] |= 128 >> (nmOnes % 8)
 		cidr := net.IPNet{IP: ip, Mask: net.CIDRMask(nmOnes+1, nmBits)}
-		cluster.Spec.KubeControllerManager.ClusterCIDR = cidr.String()
-		klog.V(2).Infof("Defaulted KubeControllerManager.ClusterCIDR to %v", cluster.Spec.KubeControllerManager.ClusterCIDR)
+		cluster.Spec.PodCIDR = cidr.String()
+		klog.V(2).Infof("Defaulted PodCIDR to %v", cluster.Spec.PodCIDR)
 	}
 
 	if cluster.Spec.ServiceClusterIPRange == "" {
 		// Allocate from the '0' subnet; but only carve off 1/4 of that (i.e. add 1 + 2 bits to the netmask)
-		cidr := net.IPNet{IP: nonMasqueradeCIDR.IP.Mask(nonMasqueradeCIDR.Mask), Mask: net.CIDRMask(nmOnes+3, nmBits)}
+		serviceOnes := nmOnes + 3
+		// Max size of network is 20 bits
+		if nmBits-serviceOnes > 20 {
+			serviceOnes = nmBits - 20
+		}
+		cidr := net.IPNet{IP: nonMasqueradeCIDR.IP.Mask(nonMasqueradeCIDR.Mask), Mask: net.CIDRMask(serviceOnes, nmBits)}
 		cluster.Spec.ServiceClusterIPRange = cidr.String()
 		klog.V(2).Infof("Defaulted ServiceClusterIPRange to %v", cluster.Spec.ServiceClusterIPRange)
 	}
